# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import textwrap
import yaml

from charmworld.models import (
    Bundle,
    CharmSource,
    construct_charm_id,
    FeaturedSource,
    QADataSource,
    store_bundles,
)
from charmworld.search import (
    BUNDLE,
    CHARM,
)
from charmworld.testing import (
    factory,
    MongoTestBase,
)
from charmworld.migrations import migrate
from charmworld.migrations.migrate import Versions
from charmworld.tests.test_search import put_mapping

import mock


class MigrationTestBase(MongoTestBase):

    def setUp(self):
        super(MigrationTestBase, self).setUp()
        self.versions = Versions(migrate.get_migration_path())
        self.use_context(mock.patch.object(migrate, 'configure_logging'))


class TestMigration018(MigrationTestBase):

    def setUp(self):
        super(TestMigration018, self).setUp()
        self.use_index_client()
        self.store_some_bundles()

    def store_some_bundles(self):
        deployer_config = textwrap.dedent("""\
            wordpress-stage:
                series: precise
                services:
                    blog:
                        charm: cs:precise/wordpress
                        constraints: mem=2
                        options:
                            tuning: optimized
                            engine: apache
        """)
        parsed = yaml.safe_load(deployer_config)
        owner = 'owner'
        bundle_name = 'wordpress-stage'
        basket_name = 'wordpress-basket'
        basket_rev = 3
        basket_id = "%s/%d" % (basket_name, basket_rev)
        self.search_id = Bundle.construct_id(
            owner, basket_name, bundle_name)
        store_bundles(self.db.bundles, parsed, owner, basket_id,
            index_client=self.index_client)

    def test_bundles_are_removed_from_elastic_search(self):
        assert self.index_client.get(self.search_id, BUNDLE), (
            'the test bundle is missing')
        self.versions.run_migration(
            self.db, self.index_client, '018_delete_all_bundles.py')
        self.assertIsNone(self.index_client.get(self.search_id, BUNDLE))

    def test_bundles_are_removed_from_mongo(self):
        assert self.index_client.get(self.search_id, BUNDLE), (
            'the test bundle is missing')
        self.versions.run_migration(
            self.db, self.index_client, '018_delete_all_bundles.py')
        self.assertIsNone(self.db.bundles.find_one({}))
